
! Copyright (C) 2016 A. Jakobsson.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

module modjx

! number of spin-spiral q-vectors for calculating the exchange coefficients
integer nqssjx
! range of cone angles for spin-spiral calculations
real(8) thssjx(2)

!**** add to manual, mention thssjx in degrees

end module

